#!/usr/bin/python
# coding=utf-8

from math import *
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mlab
import scipy as sp
import sys

C   = 1.0
n   = 9
t0  = radians(-30.0)
td  = radians(30.0)
e1  = 1.01
e2  = 2.31
e3  = 1.01
pol = 'p'
r   = sqrt(e2-e1*(sin(t0))**2)
s   = sqrt(e1)*sin(t0)
k   = 0.0; l = 0.0; u = 0.0;
if pol=='p':
	k = e1; l = e2; u = e3
elif pol=='s':
	k = 1.0; l = 1.0; u = 1.0
else:
	print 'Error in polarization'
	exit()

def rejection(p, maximum, a, b): 
	u = np.random.uniform()
	x = np.random.uniform(a, b)
	while u*1.1 > p(x)/maximum:
		u = np.random.uniform()
		x = np.random.uniform(a, b)
	return x

def f0(g):
	factor1 = 1.0/(4.0*k*u)
	factor2 = sqrt(e1/e3)
	factor3over = l*sqrt(e1)+k*sqrt(e2)
	factor3under = 2.0*l*sqrt(e1)
	
	rotTegn = 1.0 - ((e2-e3)/e3)*g**2
	if (rotTegn < 0.0):
		return 0.0
	rotTegn = sqrt(rotTegn)
	factor4over = (sqrt(e2/e3) - rotTegn)*(u + l*sqrt(e3/e2)*rotTegn)**2
	factor4under = (1.0+g**2)*rotTegn**2

	return factor1*factor2*(factor3over/factor3under)**2*factor4over/factor4under*dTdt(g)

def f(g):
	grms = g*r-s
	rpgs = r+g*s
	eng2 = 1.0+g**2
	brok = 1.0/eng2
	rotTegn = eng2*e3-grms**2
	if (rotTegn < 0.0):
		return 0.0
		print rotTegn
	rotTegn = sqrt(rotTegn)
	storSak = grms**2+rpgs*rotTegn

	prefactor = (l*sqrt(e1)*cos(t0)+k*r)**2*u/(16.0*l**2*sqrt(e1)*cos(t0)*k)
	factor1 = abs(brok*(g*grms+rotTegn)-r)**3
	factor2over = ((e2-l*e3/u)+(l/u-1.0)*brok*storSak)**2
	factor2under = ((brok*storSak-0.5*(e2+e3))**2-0.25*(e2-e3)**2)**2

	return prefactor*factor1*factor2over/factor2under*dTdt(g)

def dTdt(g):
	grms = g*r-s
	rgps = r+g*s
	eng2 = 1.0+g**2
	rotTegn = sqrt(eng2*e3-grms**2)
	
	return C*(cos(td)/(sqrt(e3)*eng2)*(g*grms+rotTegn)+sin(td)/(sqrt(e3)*eng2)*(-grms+g*rotTegn))**n

a = -3.0
b = -a

maximum = 22
N = 100000
X = np.empty(N)
for i in range(N):
	X[i] = rejection(f, maximum, a, b)
	if not i%500: print i
plt.hist(X, 100, None, True)

a = -3.0
b = -a
N = 5001
x = sp.linspace(a, b, N)
deltax = x[1]-x[0]
y1 = sp.linspace(a, b, N)
A1 = 0
maximum1 = -1

for i in range(N):
	favx = f(x[i])
	A1+=deltax*favx
	y1[i] = favx
	if (y1[i] > maximum1):
		maximum1 = y1[i]
	print x[i], y1[i]

plt.plot(x,y1/A1)
print "#A =", A1
print "#Max =", maximum1
plt.show()
